﻿//
// Copyright (C) 2009 Chris Dziemborowicz
//
// This file is part of Orzeszek Ratings.
//
// Orzeszek Ratings is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// Orzeszek Ratings is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace OrzeszekRatings
{
	/// <summary>
	/// Interaction logic for StartSyncControl.xaml
	/// </summary>
	public partial class StartSyncControl : UserControl
	{
		private SyncInfo syncInfo;

		public SettingsControl SettingsControl { get; set; }

		public event EventHandler<SyncInfoEventArgs> SyncRequested;

		public StartSyncControl()
		{
			InitializeComponent();
		}

		public void AnimateShow()
		{
			Visibility = Visibility.Visible;

			DoubleAnimationUsingKeyFrames da = new DoubleAnimationUsingKeyFrames();
			da.KeyFrames.Add(new SplineDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.Zero)));
			da.KeyFrames.Add(new SplineDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
			MainSizer.BeginAnimation(SizerControl.HeightFactorProperty, da);
		}

		public void AnimateHide()
		{
			DoubleAnimationUsingKeyFrames da = new DoubleAnimationUsingKeyFrames();
			da.KeyFrames.Add(new SplineDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.Zero)));
			da.KeyFrames.Add(new SplineDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
			da.Completed += new EventHandler(delegate(object o, EventArgs ea)
				{
					Visibility = Visibility.Collapsed;
				}
			);
			MainSizer.BeginAnimation(SizerControl.HeightFactorProperty, da);
		}

		public void Reset()
		{
			SyncFromWmpButton.IsEnabled = true;
			SyncToWmpButton.IsEnabled = true;
			ConfirmButton.IsEnabled = false;
			CancelButton.IsEnabled = false;

			ShowButtonsNow();
			HideConfirmNow();
		}

		private void SyncFromWmpButton_Click(object sender, RoutedEventArgs e)
		{
			if (!SyncPlayCountsCheckBox.IsChecked.Value && !SyncRatingsCheckBox.IsChecked.Value)
				StatusBarManager.AddMessage("You must select to sync at least the play count or the ratings.", MessageType.Error, MessageLength.Short);
			else
			{
				SyncFromWmpButton.IsEnabled = false;
				SyncToWmpButton.IsEnabled = false;
				ConfirmButton.IsEnabled = true;
				CancelButton.IsEnabled = true;

				syncInfo = new SyncInfo();
				syncInfo.Direction = SyncDirection.FromWmp;
				syncInfo.SyncPlayCounts = SyncPlayCountsCheckBox.IsChecked.Value;
				syncInfo.SyncRatings = SyncRatingsCheckBox.IsChecked.Value;

				syncInfo.ForceRatingsUpdate = SettingsControl.ForceRatingsUpdateCheckBox.IsChecked.Value;
				syncInfo.OnlySyncHighestPlayCount = SettingsControl.OnlySyncHighestPlayCountCheckBox.IsChecked.Value;
				syncInfo.AggregateMovePlayCount = SettingsControl.AggregateMovePlayCountCheckBox.IsChecked.Value;

				if (syncInfo.SyncPlayCounts && syncInfo.SyncRatings)
					ConfirmTextBlock.Text = "Are you sure you want to sync play counts and ratings from WMP to iTunes?";
				else if (syncInfo.SyncPlayCounts)
					ConfirmTextBlock.Text = "Are you sure you want to sync play counts from WMP to iTunes?";
				else
					ConfirmTextBlock.Text = "Are you sure you want to sync ratings from WMP to iTunes?";

				AnimateHideButtons();
				AnimateShowConfirm();
			}
		}

		private void SyncToWmpButton_Click(object sender, RoutedEventArgs e)
		{
			if (!SyncPlayCountsCheckBox.IsChecked.Value && !SyncRatingsCheckBox.IsChecked.Value)
				StatusBarManager.AddMessage("You must select to sync at least the play count or the ratings.", MessageType.Error, MessageLength.Short);
			else
			{
				SyncFromWmpButton.IsEnabled = false;
				SyncToWmpButton.IsEnabled = false;
				ConfirmButton.IsEnabled = true;
				CancelButton.IsEnabled = true;

				syncInfo = new SyncInfo();
				syncInfo.Direction = SyncDirection.ToWmp;
				syncInfo.SyncPlayCounts = SyncPlayCountsCheckBox.IsChecked.Value;
				syncInfo.SyncRatings = SyncRatingsCheckBox.IsChecked.Value;

				syncInfo.ForceRatingsUpdate = SettingsControl.ForceRatingsUpdateCheckBox.IsChecked.Value;
				syncInfo.OnlySyncHighestPlayCount = SettingsControl.OnlySyncHighestPlayCountCheckBox.IsChecked.Value;
				syncInfo.AggregateMovePlayCount = SettingsControl.AggregateMovePlayCountCheckBox.IsChecked.Value;

				if (syncInfo.SyncPlayCounts && syncInfo.SyncRatings)
					ConfirmTextBlock.Text = "Are you sure you want to sync play counts and ratings from iTunes to WMP?";
				else if (syncInfo.SyncPlayCounts)
					ConfirmTextBlock.Text = "Are you sure you want to sync play counts from iTunes to WMP?";
				else
					ConfirmTextBlock.Text = "Are you sure you want to sync ratings from iTunes to WMP?";

				AnimateHideButtons();
				AnimateShowConfirm();
			}
		}

		private void ConfirmButton_Click(object sender, RoutedEventArgs e)
		{
			ConfirmButton.IsEnabled = false;
			CancelButton.IsEnabled = false;

			AnimateHide();

			if (SyncRequested != null)
				SyncRequested(this, new SyncInfoEventArgs(syncInfo));
		}

		private void CancelButton_Click(object sender, RoutedEventArgs e)
		{
			SyncFromWmpButton.IsEnabled = true;
			SyncToWmpButton.IsEnabled = true;
			ConfirmButton.IsEnabled = false;
			CancelButton.IsEnabled = false;

			syncInfo = null;

			AnimateHideConfirm();
			AnimateShowButtons();
		}

		private void AnimateShowButtons()
		{
			ButtonsSizer.Visibility = Visibility.Visible;

			DoubleAnimationUsingKeyFrames da = new DoubleAnimationUsingKeyFrames();
			da.KeyFrames.Add(new SplineDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.Zero)));
			da.KeyFrames.Add(new SplineDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
			ButtonsSizer.BeginAnimation(SizerControl.HeightFactorProperty, da);
		}

		private void AnimateHideButtons()
		{
			DoubleAnimationUsingKeyFrames da = new DoubleAnimationUsingKeyFrames();
			da.KeyFrames.Add(new SplineDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.Zero)));
			da.KeyFrames.Add(new SplineDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
			da.Completed += new EventHandler(delegate(object o, EventArgs ea)
				{
					ButtonsSizer.Visibility = Visibility.Collapsed;
				}
			);
			ButtonsSizer.BeginAnimation(SizerControl.HeightFactorProperty, da);
		}

		private void AnimateShowConfirm()
		{
			ConfirmSizer.Visibility = Visibility.Visible;

			DoubleAnimationUsingKeyFrames da = new DoubleAnimationUsingKeyFrames();
			da.KeyFrames.Add(new SplineDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.Zero)));
			da.KeyFrames.Add(new SplineDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
			ConfirmSizer.BeginAnimation(SizerControl.HeightFactorProperty, da);
		}

		private void AnimateHideConfirm()
		{
			DoubleAnimationUsingKeyFrames da = new DoubleAnimationUsingKeyFrames();
			da.KeyFrames.Add(new SplineDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.Zero)));
			da.KeyFrames.Add(new SplineDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
			da.Completed += new EventHandler(delegate(object o, EventArgs ea)
				{
					ConfirmSizer.Visibility = Visibility.Collapsed;
				}
			);
			ConfirmSizer.BeginAnimation(SizerControl.HeightFactorProperty, da);
		}

		private void ShowButtonsNow()
		{
			ButtonsSizer.Visibility = Visibility.Visible;

			DoubleAnimationUsingKeyFrames da = new DoubleAnimationUsingKeyFrames();
			da.KeyFrames.Add(new SplineDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.Zero)));
			ButtonsSizer.BeginAnimation(SizerControl.HeightFactorProperty, da);
		}

		private void HideConfirmNow()
		{
			DoubleAnimationUsingKeyFrames da = new DoubleAnimationUsingKeyFrames();
			da.KeyFrames.Add(new SplineDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.Zero)));
			da.Completed += new EventHandler(delegate(object o, EventArgs ea)
			{
				ConfirmSizer.Visibility = Visibility.Collapsed;
			}
			);
			ConfirmSizer.BeginAnimation(SizerControl.HeightFactorProperty, da);
		}
	}
}